/**
* This sample shows how to use the function Read1AnalogInputInterrupt
**/

#include "stdafx.h"

/**
* Interrupt routine that can be used in the samples
* If you register this interrupt routine thanks to the function i_PCI3120_SetBoardIntRoutine, this function
* will be called each time the board generates an interruption
**/
void InterruptRoutine (
	HANDLE	deviceHandle,
	BYTE	fifoOverflow,
	BYTE	interruptMask,
	PWORD	args,
	DWORD	argsCount )
{
	UNREFERENCED_PARAMETER(deviceHandle);
	UNREFERENCED_PARAMETER(fifoOverflow);
	
	printf("interrupt mask : %d \n", (int) interruptMask );

	if ( (interruptMask & 0x1) != 0 )
	{
		printf("EOS interruption. Number of value available : %d \n", (argsCount) );
		printf("Value = 0x%x \n", (int) args[0] );
	}
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT	returnValue = 0;
	HANDLE boardHandle;
	ULONG readValue = 0;
	BYTE channel = 0;
	BYTE polarity = 0;
	BYTE gain = 0;
	BYTE inputMode = 0;
	WORD value = 0;
	char displayBuffer[2048] = {0};

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");
	printf("* This sample enables to show how the i_PCI3120_Read1AnalogInput works\n");
	printf("* It uses interrupt. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n\n");

	/* install an interrupt routine */
	returnValue = i_PCI3120_SetBoardIntRoutine(boardHandle, InterruptRoutine);
	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3120_SetBoardIntRoutine \n", returnValue);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* ask the input mode */
	ReadUlong("Select the input mode (0: Single-ended, 1: Differential) ", 0, 1, &readValue);
	switch ( readValue )
	{
		case 0:
			inputMode = PCI3120_INPUT_MODE_SINGLE_ENDED;
			break;
		case 1:	
			inputMode = PCI3120_INPUT_MODE_DIFFERENTIAL;
			break;
	}

	/* set the selected input mode */
	returnValue = i_PCI3120_SetAnalogInputMode( boardHandle, inputMode );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCi3120_SetAnalogInputMode \n", returnValue);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* ask the input channel */
	if ( inputMode == PCI3120_INPUT_MODE_SINGLE_ENDED )
	{
		sprintf_s( displayBuffer, "Select the input channel (0 to %u) ", PCI3120_ANALOG_INPUT_NUMBER-1 );
		ReadUlong(displayBuffer, 0, (PCI3120_ANALOG_INPUT_NUMBER-1), &readValue);
	}
	else
	{
		sprintf_s( displayBuffer, "Select the input channel (0 to %u) ", (PCI3120_ANALOG_INPUT_NUMBER/2)-1 );
		ReadUlong(displayBuffer, 0, (PCI3120_ANALOG_INPUT_NUMBER/2)-1, &readValue);
	}
	channel = (UCHAR) readValue ;


	/* ask the gain */
	ReadUlong("Select the input gain (0: x1, 1: x2, 2: x5, 3: x10) ", 0, 3, &readValue);
	switch ( readValue )
	{
		case 0:
			gain = PCI3120_GAIN_1;
			break;
		case 1:	
			gain = PCI3120_GAIN_2;
			break;
		case 2:	
			gain = PCI3120_GAIN_5;
			break;
		case 3:	
			gain = PCI3120_GAIN_10;
			break;
	}

	/* ask the polarity */
	ReadUlong("Select the input polarity (0: Unipolar, 1: Bipolar) ", 0, 1, &readValue);
	switch ( readValue )
	{
		case 0:	
			polarity = APCI3120_UNIPOLAR;
			break;
		case 1:
			polarity = APCI3120_BIPOLAR;
			break;
	}

	returnValue = i_PCI3120_Read1AnalogInput (
		boardHandle,
		channel,
		gain,
		polarity,
		10,								/* 10us converting time */
		1,								/* interrupt enabled */
		&value );

	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3120_Read1AnalogInput \n", returnValue);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	printf("Hit a key to exit .\n");
	_getch();


	/* close the board */
	returnValue = i_PCI3120_CloseBoard(boardHandle);
	if ( returnValue != 0 )
	{
		printf("Error while closing board : %d \n", returnValue );
		_getch();
		return -1;
	}

	WaitForESC();

	/* success */
	return 0;
}

