/**
* This sample show you how to use the ReadMoreAnalogInputs function of the board
* If you want to use high frequency, you should better use Sequence acquisition instead of this function
* This sample doesn't use interrupt to get the value. It also doesn't use delay and trigger.
**/

#include "stdafx.h"

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE boardHandle;
	int i = 0;
	char displayBuffer[2048] = {0};
	ULONG readValue = 0;
	BYTE channelsNbr = 0;
	BYTE inputMode = 0;
	BYTE channel[16] = {0};
	BYTE gain[16] = {0};
	BYTE polarity[16] = {0};
	WORD values[16];
	WORD convertingTime = 0;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");
	printf("* This sample enables to show how to use the i_PCI3120_ReadMoreAnalogInputs method \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n\n");

	/* ask the input mode */
	ReadUlong("Select the input mode (0: Single-ended, 1: Differential) ", 0, 1, &readValue);
	switch ( readValue )
	{
		case 0:
			inputMode = PCI3120_INPUT_MODE_SINGLE_ENDED;
			break;
		case 1:
			inputMode = PCI3120_INPUT_MODE_DIFFERENTIAL;
			break;
	}

	/* ask the number of channels to use */
	sprintf_s( displayBuffer, "How many channels do you want to read? (1 to %u) ", PCI3120_ANALOG_INPUT_NUMBER );
	ReadUlong(displayBuffer, 1, PCI3120_ANALOG_INPUT_NUMBER, &readValue);
	channelsNbr = (UCHAR) readValue;

	/* configure each channel */
	for ( i = 0 ; i < channelsNbr ; i++ )
	{
		printf("Sequence index #%d \n", i);

		/* ask the index of the channel */
		switch ( inputMode )
		{
			case PCI3120_INPUT_MODE_SINGLE_ENDED:
				sprintf_s( displayBuffer, "  - Select the input channel (0 to %u) ", PCI3120_ANALOG_INPUT_NUMBER-1 );
				ReadUlong(displayBuffer, 0, PCI3120_ANALOG_INPUT_NUMBER-1, &readValue);
				break;
			case PCI3120_INPUT_MODE_DIFFERENTIAL:
				sprintf_s( displayBuffer, "  - Select the input channel (0 to %u) ", (PCI3120_ANALOG_INPUT_NUMBER/2)-1 );
				ReadUlong(displayBuffer, 0, (PCI3120_ANALOG_INPUT_NUMBER/2)-1, &readValue);
				break;
		}
		channel[i] = (UCHAR) readValue ;

		/* ask the gain */
		ReadUlong("  - Select the input gain (0: x1, 1: x2, 2: x5, 3: x10) ", 0, 3, &readValue);
		switch ( readValue )
		{
			case 0:
				gain[i] = PCI3120_GAIN_1;
				break;
			case 1:
				gain[i] = PCI3120_GAIN_2;
				break;
			case 2:
				gain[i] = PCI3120_GAIN_5;
				break;
			case 3:	
				gain[i] = PCI3120_GAIN_10;
				break;
		}

		/* ask the polarity */
		ReadUlong("  - Select the input polarity (0: Unipolar, 1: Bipolar) ", 0, 1, &readValue);
		switch ( readValue )
		{
			case 0:
				polarity[i] = APCI3120_UNIPOLAR;
				break;
			case 1:
				polarity[i] = APCI3120_BIPOLAR;
				break;
		}
	}

	/* converting time */
	ReadUlong("Select the converting time in us (10 to 32767) ", 10, 32767, &readValue);
	convertingTime = (WORD)readValue;

	/* Set the analog input mode */
	returnValue = i_PCI3120_SetAnalogInputMode( boardHandle, inputMode );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_SetAnalogInputMode \n", returnValue);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	printf("Hit a key to exit the sample\n");
	/* call the read more analog inputs method each 100ms */
	do
	{
		returnValue = i_PCI3120_ReadMoreAnalogInputs (boardHandle,
			channelsNbr,
			channel,
			gain,
			polarity,
			convertingTime,
			0,								/* no interrupt */
			values );

		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3120_ReadMoreAnalogInputs \n", returnValue);
			i_PCI3120_CloseBoard(boardHandle);
			_getch();
			return -1;
		}

		/* display values */
		printf("Channel values are ");
		for ( i = 0 ; i < channelsNbr ; i++ )
		{
			printf("0x%.4x\t", values[i]);
		}
		DisplayRunningStatus();

		Sleep(100);
	}
	while( !_kbhit() );

	/* close the board */
	returnValue = i_PCI3120_CloseBoard(boardHandle);
	if ( returnValue != 0 )
	{
		printf("Error while closing board : %d \n", returnValue );
		_getch();
		return -1;
	}

	WaitForESC();

	/* success */
	return 0;
}

