/**
* Demonstrate how to configure, start and get the value of a timer
**/
#include "stdafx.h"

#define TIMER_RELOAD_VALUE 65500 // 65500us
DWORD maxJitterValue = 0;

/**
* Interrupt routine
**/
static void TimerInterruptJitterMeasureRoutine (
	HANDLE	deviceHandle,
	BYTE	fifoOverflow,
	BYTE	interruptMask,
	PWORD	args,
	DWORD	argsCount )
{
	DWORD value = 0;
	INT	returnValue = 0;
	DWORD jitterValue = 0;

	UNREFERENCED_PARAMETER(args);
	UNREFERENCED_PARAMETER(argsCount);
	UNREFERENCED_PARAMETER(deviceHandle);
	UNREFERENCED_PARAMETER(fifoOverflow);
	UNREFERENCED_PARAMETER(interruptMask);
	//printf("Interrupt, mask is : %x \n", interruptMask );
	returnValue = i_PCI3120_ReadTimerValue(deviceHandle, &value);
	jitterValue = TIMER_RELOAD_VALUE-value;

	if(jitterValue > maxJitterValue)
	{
		if (jitterValue == 0)
			printf("Timer Value : %lu jitter < 50 us\n",value); 
		else
			printf("Timer Value : %lu jitter = %d us\n",value, (ULONG)(TIMER_RELOAD_VALUE-value)); 
	}

	
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT	returnValue = 0;
	HANDLE boardHandle = NULL;
	DWORD delay = 0;
	
	
	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* This sample will initialize a timer, with interrupt. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* set the board interrupt routine */
	returnValue = i_PCI3120_SetBoardIntRoutine( boardHandle, TimerInterruptJitterMeasureRoutine );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_SetBoardIntRoutine \n", returnValue);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* set the delay  TIMER_RELOAD_VALUE */ 
	delay = TIMER_RELOAD_VALUE;

	/* get the max jitter value */
	ReadUlong("Max jitter to test in us (50 to TIMER_RELOAD_VALUE) ",50,TIMER_RELOAD_VALUE,&maxJitterValue);
	
	/* start the timer */
	returnValue = i_PCI3120_InitAndStartTimerWatchdog( boardHandle,
		APCI3120_TIMER,
		delay,
		APCI3120_ENABLE); /* with interrupt */

	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3120_InitAndStartTimerWatchdog \n", returnValue);
		i_PCI3120_ResetBoardIntRoutine(boardHandle);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3120_InitAndStartTimerWatchdog OK \n");
	}

	printf("Press a key to stop the sample...\n");
	/* endless loop */
	for(;;)
	{
		/* does the user hit a key ? */
		if ( _kbhit() )
		{
			_getch();
			break;
		}

		Sleep(100);
	}

	/* stop the timer */
	returnValue = i_PCI3120_StopTimerWatchdog(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_StopTimerWatchdog \n", returnValue);
	}
	else
	{
		printf("i_PCI3120_StopTimerWatchdog OK \n");
	}

	/* reset the interrupt routine */
	returnValue = i_PCI3120_ResetBoardIntRoutine( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_ResetBoardIntRoutine \n", returnValue);
	}
	else
	{
		printf("i_PCI3120_ResetBoardIntRoutine OK \n");
	}

	/* close the board */
	returnValue = i_PCI3120_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCI3120_CloseBoard OK \n");
	}

	WaitForESC();

	return 0;
}

