/**
* Demonstrate how to configure, start and get the value of a timer
**/
#include "stdafx.h"

int watchdogExpired = 0;

/**
* Interrupt routine
**/
static void WatchdogInterruptRoutine (
	HANDLE	deviceHandle,
	BYTE	fifoOverflow,
	BYTE	interruptMask,
	PWORD	args,
	DWORD	argsCount )
{
	UNREFERENCED_PARAMETER(args);
	UNREFERENCED_PARAMETER(argsCount);
	UNREFERENCED_PARAMETER(deviceHandle);
	UNREFERENCED_PARAMETER(fifoOverflow);
	printf("Interrupt, mask is : %x \n", interruptMask );

	/* watchdog expired */
	if ( (interruptMask & 0x20) == 0x20 )
	{
		watchdogExpired = 1;
	}
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT	returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;
	DWORD delay = 0;
	
	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* This sample will initialize a watchdog, with interrupt. \n");
	printf("* Hit the space bar to write on the analog output 0. Press 'ESC' to \n");
	printf("* exit this sample. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* set the board interrupt routine */
	returnValue = i_PCI3120_SetBoardIntRoutine( boardHandle, WatchdogInterruptRoutine );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_SetBoardIntRoutine \n", returnValue);
		i_PCI3120_ResetBoardIntRoutine(boardHandle);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* get the delay */
	ReadUlong("Delay you want to use, in us (50 to 838860850) ",50,838860850,&readValue);
	delay = readValue;

	/* start the timer */
	returnValue = i_PCI3120_InitAndStartTimerWatchdog( boardHandle,
		APCI3120_WATCHDOG,
		delay,
		APCI3120_ENABLE); /* with interrupt */

	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3120_InitAndStartTimerWatchdog \n", returnValue);
		i_PCI3120_ResetBoardIntRoutine(boardHandle);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3120_InitAndStartTimerWatchdog OK \n");
	}

	/* write one analog output */
	returnValue = i_PCI3120_Write1AnalogOutput( boardHandle, 
		0, 
		APCI3120_UNIPOLAR, 
		0xFFF );
	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3120_Write1AnalogOutput \n", returnValue);
		i_PCI3120_ResetBoardIntRoutine(boardHandle);
		i_PCI3120_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3120_Write1AnalogOutput OK \n");
	}

	/* endless loop */
	for(;;)
	{
		BYTE status = 0;

		returnValue = i_PCI3120_ReadWatchdogStatus(boardHandle, &status);
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3120_ReadWatchdogStatus \n", returnValue);
			i_PCI3120_ResetBoardIntRoutine(boardHandle);
			i_PCI3120_CloseBoard(boardHandle);
			_getch();
			return -1;
		}

		/* display the value */
		printf("Status is : %x \t \t \r", status );

		/* does the watchdog expires ? */
		if ( watchdogExpired == 1 )
		{
			printf("Watchdog has expired\n");
			break;
		}

		/* does the user hit a key ? */
		if ( _kbhit() )
		{
			int hitChar = _getch();
			if ( hitChar == 27 )
			{
				/* 'ESC' */
				break;
			}
			else if ( hitChar == 32 )
			{
				/* 'space' 
				 * write one analog output
				 */
				returnValue = i_PCI3120_Write1AnalogOutput( boardHandle, 
					0, 
					APCI3120_UNIPOLAR, 
					0xFFF );
				if ( returnValue != 0 )
				{
					printf("Error %d in i_PCI3120_Write1AnalogOutput \n", returnValue);
					i_PCI3120_ResetBoardIntRoutine(boardHandle);
					i_PCI3120_CloseBoard(boardHandle);
					_getch();
					return -1;
				}
				else
				{
					printf("i_PCI3120_Write1AnalogOutput OK \n");
				}
			}
		}

		Sleep(100);
	}

	/* stop the timer */
	returnValue = i_PCI3120_StopTimerWatchdog(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_StopTimerWatchdog \n", returnValue);
	}
	else
	{
		printf("i_PCI3120_StopTimerWatchdog OK \n");
	}

	/* reset the interrupt routine */
	returnValue = i_PCI3120_ResetBoardIntRoutine( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_ResetBoardIntRoutine \n", returnValue);
	}
	else
	{
		printf("i_PCI3120_ResetBoardIntRoutine OK \n");
	}

	/* close the board */
	returnValue = i_PCI3120_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3120_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCI3120_CloseBoard OK \n");
	}

	WaitForESC();
	 
	return 0;
}

