using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3120_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            byte mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_SetAnalogInputMode(IntPtr deviceHandle, byte inputMode);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_GetAnalogInputMode(IntPtr deviceHandle, ref byte inputMode);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_InitAndStartAnalogInputSequence(IntPtr deviceHandle,
            uint sequenceCount, 
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            short convertingTime,
            byte delayMode,
            uint delayTime,
            byte extTriggerMode);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_StopAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCI3120.dll")]
        public static extern int l_PCI3120_ReadSequenceData(IntPtr deviceHandle, uint size, ushort[] data);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_ReadMoreAnalogInputs(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            ushort convertingTime,
            byte interruptFlag,
            ushort[] value);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_Write1AnalogOutput(IntPtr deviceHandle,
            byte channelIndex,
            byte polarity,
            ushort value);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_WriteMoreAnalogOutputs(IntPtr deviceHandle, 
            byte channelNbr,
            byte[] channelArray,
            byte[] polarityArray,
            ushort[] value);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_Set4DigitalOutputsOff(IntPtr deviceHandle, byte channelIndex);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_Set4DigitalOutputsOn(IntPtr deviceHandle, byte channelIndex);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_Get4DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_InitAndStartTimerWatchdog(IntPtr deviceHandle, byte timerMode, uint delayValue, byte interruptFlag);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_StopTimerWatchdog(IntPtr deviceHandle);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_ReadWatchdogStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_ReadTimerValue(IntPtr deviceHandle, ref uint value);

        [DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_WriteTimerValue(IntPtr deviceHandle, uint value);
		
		[DllImport("PCI3120.dll")]
        public static extern short i_PCI3120_GetBoardInformation(Byte index,
			UInt32 identifierStringSize,
	        Byte[] identifier,
	        out UInt32 uiNumber,
            out UInt32 deviceNumber,
            out UInt32 busNumber,
            out UInt32 baseAddress0,
            out UInt32 baseAddress1,
            out UInt32 baseAddress2,
			out UInt32 FW_Version,
			out Byte   interrupt);
    }
}
