using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TimerInterrupt
{
    public partial class TimerInterrupt : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCI3120_NET.PCI3120 board;

        /// <summary>
        /// Some internal value. Store the number of interrupt per component
        /// </summary>
        private int timerInterruptNb = 0;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public TimerInterrupt()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TimerInterrupt_Load(object sender, EventArgs e)
        {
            int ret = 0;
            try
            {
                this.board = PCI3120_NET.PCI3120.OpenBoard(0);
                if (board == null)
                {
                    MessageBox.Show("Error while opening board 0");
                    this.Close();
                    return;
                }
            }
            catch (Exception)
            {
                MessageBox.Show("Unable to load the APCI-1516 driver DLL!");
                this.Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                this.Close();
                return;
            }

            // Stop the timer watchdog 0
            this.board.StopTimerWatchdog();

            // Listen to the counter timer watchdog interruption
            this.board.TimerWatchdogEventReceived += new PCI3120_NET.PCI3120.TimerWatchdogEventHandler(board_TimerWatchdogEventReceived);
        }

        /// <summary>
        /// Interrupt received
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_TimerWatchdogEventReceived(object source, PCI3120_NET.TimerWatchdogEventArgs args)
        {
            this.timerInterruptNb++;
        }

        /// <summary>
        /// Display timer for timer 1 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            uint value = 0;

            // Read the value
            if ((ret = this.board.ReadTimerValue(ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadTimerWatchdogValue");
                return;
            }
            timer1Value.Text = value.ToString();
            timer1Interrupt.Text = this.timerInterruptNb.ToString();
        }

        /// <summary>
        /// Click on the start button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Start_Click(object sender, EventArgs e)
        {
            int ret = 0;
            UD_ReloadValue.Value = ((((UInt32)UD_ReloadValue.Value + 25) / 50) * 50);
            
             // Initialize the component as a timer
            if ((ret = this.board.InitAndStartTimerWatchdog(
                60,  // timer mode
                Convert.ToUInt32(UD_ReloadValue.Value),  // reload time 
                1)) // interrupt
                != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartTimerWatchdog");
                return;
            }

            // Disable/enable button
            UD_ReloadValue.Enabled = false;
            timer1Start.Enabled = false;
            timer1Stop.Enabled = true;

            // Start the display timer
            timer1DisplayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button for the timer 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Stop the timer
            if ((ret = this.board.StopTimerWatchdog()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopTimerWatchdog");
                return;
            }
            // Disable/enable button
            UD_ReloadValue.Enabled = true;
            timer1Start.Enabled = true;
            timer1Stop.Enabled = false;

            // Stop the display timer
            timer1DisplayTimer.Stop();
        }


        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TimerInterrupt_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;

            // Stop the timer
            this.board.StopTimerWatchdog();

            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }
    }
}