Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms


Public Class Form1
    'Handle to the first board
    Dim board As PCI3120_NET.PCI3120

    'Useful variable that enables us to make easy access on the channels boxes
    Dim channelsBox(15) As CheckBox

    'List of selected channels
    Dim channelArray() As Byte

    'The number of analog input
    Dim inputCount As Byte = 16

    'Delegate that enables to set the text of the text box from another thread
    Public Delegate Sub SetTextDelegate(ByVal text As String)



    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim ret As New Integer
        Dim returnValue As Integer = 0
        StopButton.Enabled = False

        'Open the first board of the system
        board = PCI3120_NET.PCI3120.OpenBoard(0)
        'Error handling
        If (IsNothing(board)) Then
            MessageBox.Show("Error OpenBoard returned null")
            Close()
            Exit Sub
        End If

        'Create the useful array
        Me.channelsBox(0) = Me.channel0
        Me.channelsBox(1) = Me.channel1
        Me.channelsBox(2) = Me.channel2
        Me.channelsBox(3) = Me.channel3
        Me.channelsBox(4) = Me.channel4
        Me.channelsBox(5) = Me.channel5
        Me.channelsBox(6) = Me.channel6
        Me.channelsBox(7) = Me.channel7
        Me.channelsBox(8) = Me.channel8
        Me.channelsBox(9) = Me.channel9
        Me.channelsBox(10) = Me.channel10
        Me.channelsBox(11) = Me.channel11
        Me.channelsBox(12) = Me.channel12
        Me.channelsBox(13) = Me.channel13
        Me.channelsBox(14) = Me.channel14
        Me.channelsBox(15) = Me.channel15

        'Enable the interruption
        ret = Me.board.SetBoardIntRoutine()
        If (ret <> 0) Then
            MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine")
            Close()
            Exit Sub
        End If

        AddHandler board.EndOfSequenceEventReceived, New PCI3120_NET.PCI3120.EndOfSequenceEventHandler(AddressOf board_EndOfSequenceEventReceived)

    End Sub

    'Get the end of sequence interrupt
    Private Sub board_EndOfSequenceEventReceived(ByVal source As Object, ByVal args As PCI3120_NET.EndOfSequenceEventArgs)
        Dim text As String = "Interrupt" + Environment.NewLine
        Dim i As Integer = 0

        For i = 0 To args.ArgsSize - 1

            text = text + "Channel " + Me.channelArray(i).ToString + " = " + args.Args(i).ToString() + Environment.NewLine

        Next i

        Me.resultBox.Invoke(New SetTextDelegate(AddressOf SetText), text)

    End Sub


    'Set the current text of the result box
    'Called by other thread generally to signal that read is finished

    Private Sub SetText(ByVal text As String)
        Me.resultBox.Text = text

        'Stop the progress bar
        Me.progressBar.Style = ProgressBarStyle.Blocks

        If (Me.yesRadio.Checked = True) Then
            'Enable to read again
            Me.readButton.Enabled = True
        End If


    End Sub

    'Occurs while the form is closing 
    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        Dim ret As Integer
        ret = Me.board.ResetBoardIntRoutine()
        'Disable the interruption
        If (ret <> 0) Then
            MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine")
            Exit Sub
        End If

    End Sub

    'Select / Deselect all the channels
    Private Sub selectDeselectBox_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles selectDeselectBox.CheckedChanged
        Dim i As Integer

        'Select - deselect all
        For i = 0 To Me.inputCount - 1

            Me.channelsBox(i).Checked = Me.selectDeselectBox.Checked
        Next i
    End Sub

    'Check and return the correct value of the converting time parameter
    Private Function checkAndGetConvertingTime(ByVal convertingTime As UInteger) As Boolean
        Try
            convertingTime = Convert.ToUInt32(Me.convertingTimeText.Text)
            If (convertingTime < 10) Then
                convertingTime = 0
                MessageBox.Show("Please set the converting time between 10 and 32767")
                Return False
            Else
                If (convertingTime > 32767) Then
                    convertingTime = 0
                    MessageBox.Show("Please set the converting time between 10 and 32767")
                    Return False
                End If
            End If
            Return True

        Catch ex As Exception
            convertingTime = 0
            Return False
        End Try
    End Function

    'Return the number of selected channels
    Private Function getSelectChannelsCount() As UInteger
        Dim count As UInteger = 0
        Dim i As Integer

        For i = 0 To channelsBox.Length - 1
            If (channelsBox(i).Checked = True) Then
                count = count + 1
            End If
        Next i
        Return count
    End Function

    'Construct the channel array
    Private Function getChannelArray() As Byte()
        Dim currentIndex As Integer = 0
        Dim array(getSelectChannelsCount() - 1) As Byte
        Dim i As Integer

        For i = 0 To Me.inputCount - 1
            If (Me.channelsBox(i).Checked = True) Then
                array(currentIndex) = Convert.ToByte(i)
                currentIndex = currentIndex + 1
            End If
        Next i

        Return array
    End Function

    'Click on the read button
    Private Sub readButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles readButton.Click
        Dim convertingTime As UInteger = 0
        Dim returnValue As Integer = 0
        Dim channelCount As UInteger = getSelectChannelsCount()
        convertingTime = Convert.ToUInt32(Me.convertingTimeText.Text)

        Me.resultBox.Text = ""
        Me.resultBox.Update()

        If (channelCount = 0) Then
            MessageBox.Show("Select at least one channel")
            Exit Sub
        End If

        'Check the analog input mode
        If (seRadio.Checked = False) Then
            If (diffRadio.Checked = False) Then
                MessageBox.Show("Please choose the input mode")
                Exit Sub
            End If
        End If

        'Check the interrupt mode
        If (yesRadio.Checked = False) Then
            If (noRadio.Checked = False) Then
                MessageBox.Show("Please choose the interrupt mode")
                Exit Sub
            End If
        End If

        If (Unipolar.Checked = False) Then
            If (Bipolar.Checked = False) Then
                MessageBox.Show("Please choose the polar mode")
                Exit Sub
            End If
        End If

        If (Me.checkAndGetConvertingTime(convertingTime) = False) Then
            MessageBox.Show("Converting time : invalid value")
            Exit Sub
        End If

        StopButton.Enabled = True
        Me.channelSelectionBox.Enabled = False
        Me.sequenceBox.Enabled = False

        'Construct the gain and polarity array
        Dim gainArray(Me.inputCount - 1) As Byte
        Dim polarityArray(Me.inputCount - 1) As Byte
        Dim i As Integer
        Dim values(channelCount - 1) As UShort


        For i = 0 To Me.inputCount - 1
            gainArray(i) = 0 'gain 1
            If (Unipolar.Checked = True) Then
                polarityArray(i) = 1 'unipolar
            Else
                polarityArray(i) = 0 'bipolar
            End If
        Next i

        'Construct the channel array
        Me.channelArray = getChannelArray()

        'Start progress bar and disable read button
        Me.progressBar.Style = ProgressBarStyle.Marquee
        Me.readButton.Enabled = False

        'Now two cases, interrupt or not ...
        If (Me.yesRadio.Checked = True) Then

            'Interrupt   
            returnValue = Me.board.ReadMoreAnalogInputs(Convert.ToByte(channelCount), channelArray, gainArray, polarityArray, convertingTime, 1, values)

            If (returnValue <> 0) Then
                'Error, set the text
                Me.SetText("ReadMoreAnalogInputs error : " + returnValue.ToString())
                Exit Sub
            End If

        Else
            'No interrupt
            'Use a background worker to not block the graphic interface
            Dim configurate As New Configuration
            configurate.channelArray = channelArray
            configurate.convertingTime = convertingTime
            configurate.gainArray = gainArray
            configurate.polarityArray = polarityArray
            configurate.channelCount = Convert.ToByte(channelCount)

            'Start the worker
            Me.readWithoutInterruptWorker.WorkerSupportsCancellation = True
            Me.readWithoutInterruptWorker.RunWorkerAsync(configurate)
        End If

    End Sub


    'SE mode is set
    Private Sub seRadio_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles seRadio.CheckedChanged
        If (Me.seRadio.Checked) Then
            Dim returnValue As Integer
            returnValue = Me.board.SetAnalogInputMode(1)
            If (returnValue <> 0) Then
                MessageBox.Show("SetAnalogInputMode error : " + returnValue.ToString())
                Exit Sub
            End If

            Me.channel8.Enabled = True
            Me.channel9.Enabled = True
            Me.channel10.Enabled = True
            Me.channel11.Enabled = True
            Me.channel12.Enabled = True
            Me.channel13.Enabled = True
            Me.channel14.Enabled = True
            Me.channel15.Enabled = True

        End If

    End Sub

    'DIFF mode is set
    Private Sub diffRadio_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles diffRadio.CheckedChanged
        If (Me.diffRadio.Checked) Then
            Dim returnValue As Integer
            returnValue = Me.board.SetAnalogInputMode(2)
            If (returnValue <> 0) Then
                MessageBox.Show("SetAnalogInputMode error : " + returnValue.ToString())
                Exit Sub
            End If

            Me.channel8.Checked = False
            Me.channel9.Checked = False
            Me.channel10.Checked = False
            Me.channel11.Checked = False
            Me.channel12.Checked = False
            Me.channel13.Checked = False
            Me.channel14.Checked = False
            Me.channel15.Checked = False

            Me.channel8.Enabled = False
            Me.channel9.Enabled = False
            Me.channel10.Enabled = False
            Me.channel11.Enabled = False
            Me.channel12.Enabled = False
            Me.channel13.Enabled = False
            Me.channel14.Enabled = False
            Me.channel15.Enabled = False


        End If
    End Sub


    'Do the job

    Private Sub readWithoutInterruptWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles readWithoutInterruptWorker.DoWork
        Dim returnValue As Integer = 0
        Dim configuration As Configuration = e.Argument
        Dim values(configuration.channelCount - 1) As UShort
        Me.readButton.Enabled = False


        While (Me.StopButton.Enabled)
            Dim text As String = String.Empty
            'Interrupt
            returnValue = Me.board.ReadMoreAnalogInputs(configuration.channelCount, configuration.channelArray, configuration.gainArray, configuration.polarityArray, configuration.convertingTime, 0, values)

            If (returnValue <> 0) Then
                'Error, set the text
                Me.resultBox.Invoke(New SetTextDelegate(AddressOf SetText), "ReadMoreAnalogInputs error : " + returnValue.ToString())
                Exit Sub
            End If

            'Ok display the values
            Dim i As Integer

            For i = 0 To Convert.ToInt32(configuration.channelCount) - 1
                text = text + "Channel " + configuration.channelArray(i).ToString() + " = " + values(i).ToString() + Environment.NewLine
            Next i
            Me.resultBox.Invoke(New SetTextDelegate(AddressOf SetText), text)

        End While

    End Sub

    Private Sub StopButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StopButton.Click
        StopButton.Enabled = False
        If (readWithoutInterruptWorker.IsBusy = True) Then
            readWithoutInterruptWorker.CancelAsync()
        End If

        readButton.Enabled = True

        Me.resultBox.Text = ""
        Me.channelSelectionBox.Enabled = True
        Me.sequenceBox.Enabled = True

    End Sub

    Private Sub readWithoutInterruptWorker_RunWorkerCompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles readWithoutInterruptWorker.RunWorkerCompleted
        Me.resultBox.Text = ""
    End Sub

    Private Sub readWithoutInterruptWorker_ProgressChanged(ByVal sender As System.Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles readWithoutInterruptWorker.ProgressChanged
        Me.resultBox.Text = ""
    End Sub
End Class

Public Class Configuration
    Public convertingTime As Integer
    Public gainArray() As Byte
    Public polarityArray() As Byte
    Public channelArray() As Byte
    Public channelCount As Byte

End Class